from typing import Callable
import asyncio
import logging


class async_runner:
    on_closes = []

    def exeute_on_close(self):
        try:
            for fn in self.on_closes:
                try:
                    fn()
                except KeyboardInterrupt:
                    continue
                except BaseException as e:
                    logging.exception(e.with_traceback(None), stack_info=True)
        except:
            pass

    def on_close_function(self, fn: Callable):
        self.on_closes.append(fn)
        return fn

    async def close_check_task(self):
        while True:
            try:
                has_no_finished_task = False
                cur_task = asyncio.current_task(self.loop)
                for task in asyncio.all_tasks(self.loop):
                    if cur_task == task or task.done():
                        continue
                    has_no_finished_task = True
                    break

                if not has_no_finished_task:
                    return

                await asyncio.sleep(1, loop=self.loop)
            except:
                pass

    def run_forever(self, loop: asyncio.BaseEventLoop = asyncio.get_event_loop()):
        self.loop = loop
        try:
            self.loop.run_forever()
        except KeyboardInterrupt:
            self.exeute_on_close()
            self.loop.run_until_complete(self.close_check_task())
            self.loop.stop()
            self.loop.close()
        except BaseException as e:
            logging.exception(e.with_traceback(None), stack_info=True)
            return


g_async_runner = async_runner()


def on_close_function(fn: Callable):
    return g_async_runner.on_close_function(fn)



import asyncio
import sys
import os

filepath = os.path.split(__file__)[0]
sys.path.append(".")


from l0n0lutils.async_runner import async_runner


async def task_test():
    await asyncio.sleep(3)
    print("closed")

def on_close():
    print("xxxxxx")

loop = asyncio.get_event_loop()
r = async_runner(loop)
r.on_close_function(on_close)
loop.create_task(task_test())
r.run_forever()




import setuptools

# 读取项目的readme介绍
with open("README.md", "r", encoding="utf8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="l0n0lutils",  # 项目名称，保证它的唯一性，不要跟已存在的包名冲突即可
    version="1.0.6",
    author="l0n0l",  # 项目作者
    author_email="1038352856@qq.com",
    description="一些好用的函数和类",  # 项目的一句话描述
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitee.com/l00n00l/l0n0lutils",  # 项目地址
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "xlrd==1.2.0",
        "pyyaml"
    ],
    entry_points={
        "console_scripts": [
        ]
    }
)



from multiprocessing import get_context, Process, Queue
from typing import Callable, List

class MutiProcessRunner:
    def __init__(self, process_count: int, id_start: int, id_end: int) -> None:
        self.process_count = process_count
        self.id_start = id_start
        self.id_end = id_end
        self.data_per_process = int(
            (self.id_end - self.id_start) / self.process_count)
        self.ctx = get_context("spawn")
        self.queue: Queue = self.ctx.Queue()
        self.processes: List[Process] = []

    def run(self, fn: Callable, *args, **kwargs):
        for i in range(self.process_count):
            p_id_start = self.data_per_process * i + self.id_start
            if i != self.process_count - 1:
                p_id_end = p_id_start + self.data_per_process
            else:
                p_id_end = self.id_end
            p = self.ctx.Process(target=fn,
                                 args=(p_id_start,
                                       p_id_end,
                                       self.queue,
                                       *args), kwargs=kwargs)
            p.start()
            self.processes.append(p)

    def join(self):
        for p in self.processes:
            p.join()

    def terminate(self):
        for p in self.processes:
            p.terminate()

    def kill(self):
        for p in self.processes:
            p.kill()

